#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Sep  9 08:36:33 2018
Machine learning practical 1 part 2
@author: thierrychateau
"""
import os
import numpy as np
from matplotlib import pyplot as plt
from matplotlib.colors import ListedColormap
from scipy.stats import multivariate_normal
from mpl_toolkits.mplot3d import Axes3D
from sklearn.neighbors import KernelDensity
import cv2
#import pdb; pdb.set_trace()

# plot KDE based decision function 
def plotKDEdecision(kde1,kde2, Xdata1, Xdata2,x_min=-1, x_max=1, y_min=-1, y_max=1, res=300\
             , title = 'KDE descision function for class 1'):
    #Create grid and multivariate KDE estimation
    x = np.linspace(x_min,x_max,res)
    y = np.linspace(y_min,y_max,res)
    X, Y = np.meshgrid(x,y)
    pos = np.empty(X.shape + (2,))
    pos[:, :, 0] = X; pos[:, :, 1] = Y
    log_dens1 = kde1.score_samples(np.transpose(np.array([X.flatten(),Y.flatten()])))
    log_dens1 = log_dens1.reshape(X.shape[0],X.shape[1])
    log_dens2 = kde2.score_samples(np.transpose(np.array([X.flatten(),Y.flatten()])))
    log_dens2 = log_dens2.reshape(X.shape[0],X.shape[1])
    #pdb.set_trace()
    #Make a 3D plot
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.plot_surface(X, Y, \
        np.exp(log_dens1)/(np.exp(log_dens1)+np.exp(log_dens2)),\
        cmap='viridis',linewidth=0,vmin=0,vmax=1)
    ax.scatter(Xdata1[:,0],Xdata1[:,1],label='Class 1')
    ax.scatter(Xdata2[:,0],Xdata2[:,1],label='Class 2')
    ax.set_xlabel('X axis')
    ax.set_ylabel('Y axis')
    ax.set_zlabel('Z axis')
    ax.legend()
    ax.set_title(title)


# plot KDE estimation 
def plotKDE(kde, Xdata, x_min=-1, x_max=1, y_min=-1, y_max=1, res=300\
             , title = 'KDE pdf'):
    #Create grid and multivariate KDE estimation
    x = np.linspace(x_min,x_max,res)
    y = np.linspace(y_min,y_max,res)
    X, Y = np.meshgrid(x,y)
    pos = np.empty(X.shape + (2,))
    pos[:, :, 0] = X; pos[:, :, 1] = Y
    log_dens = kde.score_samples(np.transpose(np.array([X.flatten(),Y.flatten()])))
    log_dens = log_dens.reshape(X.shape[0],X.shape[1])
    #pdb.set_trace()
    #Make a 3D plot
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.plot_surface(X, Y, np.exp(log_dens),cmap='viridis',linewidth=0)
    ax.scatter(Xdata[:,0],Xdata[:,1])
    ax.set_xlabel('X axis')
    ax.set_ylabel('Y axis')
    ax.set_zlabel('Z axis')
    ax.set_title(title)

def getfeatures (path, cl):
    X = []
    scl = 'c'+str(cl)
    for i in os.listdir(path):
        print("processing file ",i)
        if os.path.isfile(os.path.join(path,i)) and scl in i:
            X.append(getfeature(path,i))
    return np.asarray(X)
def getfeature(path,filename):
    im = cv2.imread(path+filename)
    imrgb = cv2.cvtColor(im, cv2.COLOR_BGR2RGB)
    R = imrgb[:,:,0]
    G = imrgb[:,:,1]
    B = imrgb[:,:,2]
    # normalize
    In= R+G+B
    mIn = np.max((In.flatten()))
    Rn = R/mIn
    Gn = G/mIn
    Bn = B/mIn
    # Compute mean of each component
    X = [np.mean(Rn.flatten()),np.mean(Gn.flatten()),np.mean(Bn.flatten())]
    return X

def main():
    training_images_path ='images/dataset2/app/'
    testing_images_path ='images/dataset2/test/'
    X1 = getfeatures(training_images_path, 1)
    # Keep 2 first components
    X1 = X1[:,0:2]
    X2 = getfeatures(training_images_path, 2)
    # Keep 2 first components
    X2 = X2[:,0:2]
    Xtest1 = getfeatures(testing_images_path, 1)
    Xtest1 = Xtest1[:,0:2]
    Xtest2 = getfeatures(testing_images_path, 2)
    Xtest2 = Xtest2[:,0:2]
    # train a classifier
    kde1 = KernelDensity(kernel='gaussian', bandwidth=0.25).fit(X1)
    kde2 = KernelDensity(kernel='gaussian', bandwidth=0.25).fit(X2)

    #pdb.set_trace()
    # display
    plt.plot(X1[:,0],X1[:,1],'+',label='Class 1')
    plt.plot(X2[:,0],X2[:,1],'o',label = 'Class2')
    plt.legend()
    plt.title('Training dataset: in feature space')
    plotKDE(kde1, X1,x_min=0, y_min=0, title='KDE pdf for Class 1')
    plt.plot
    plotKDE(kde2, X2,x_min=0, y_min=0, title='KDE pdf for Class 2')
    plotKDEdecision(kde1,kde2,X1,X2,x_min=0, y_min=0)
    plt.show()
if __name__ == "__main__":
    # execute only if run as a script
    main()
    

