#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Sep  9 08:36:33 2018
Machine learning practical 1 part 2
@author: thierrychateau
"""
import os
import numpy as np
from matplotlib import pyplot as plt
from matplotlib.colors import ListedColormap
from scipy.stats import multivariate_normal
from mpl_toolkits.mplot3d import Axes3D
from sklearn.neighbors import KernelDensity
from sklearn.neighbors import NearestNeighbors
import cv2
#import pdb; pdb.set_trace()

# plot KNN posterior 
def plotKNNdecision(knn1,knn2, Xdata1, Xdata2,x_min=-1, x_max=1, y_min=-1, y_max=1, res=300\
             , title = 'KNN descision function for class 1'):
    #Create grid and multivariate KDE estimation
    x = np.linspace(x_min,x_max,res)
    y = np.linspace(y_min,y_max,res)
    X, Y = np.meshgrid(x,y)
    pos = np.empty(X.shape + (2,))
    pos[:, :, 0] = X; pos[:, :, 1] = Y
    distances1, indices = knn1.kneighbors(np.transpose(np.array([X.flatten(),Y.flatten()])))
    distances1 = np.mean(distances1,axis=1)
    distances1 = distances1.reshape(X.shape[0],X.shape[1])
    distances2, indices = knn2.kneighbors(np.transpose(np.array([X.flatten(),Y.flatten()])))
    distances2 = np.mean(distances2,axis=1)
    distances2 = distances2.reshape(X.shape[0],X.shape[1])
 
    #pdb.set_trace()
    #Make a 3D plot
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.plot_surface(X, Y, \
        (1/distances1)/(1/distances1+1/distances2),\
        cmap='viridis',linewidth=0,vmin=0,vmax=1)
    ax.scatter(Xdata1[:,0],Xdata1[:,1],label='Class 1')
    ax.scatter(Xdata2[:,0],Xdata2[:,1],label='Class 2')
    ax.set_xlabel('X axis')
    ax.set_ylabel('Y axis')
    ax.set_zlabel('Z axis')
    ax.legend()
    ax.set_title(title)


# plot KDE estimation 
def plotKNN(kde, Xdata, x_min=-1, x_max=1, y_min=-1, y_max=1, res=300\
             , title = 'KDE pdf'):
    #Create grid and multivariate KDE estimation
    x = np.linspace(x_min,x_max,res)
    y = np.linspace(y_min,y_max,res)
    X, Y = np.meshgrid(x,y)
    pos = np.empty(X.shape + (2,))
    pos[:, :, 0] = X; pos[:, :, 1] = Y
    distances, indices = kde.kneighbors(np.transpose(np.array([X.flatten(),Y.flatten()])))
    distances = np.mean(distances,axis=1)
    distances = distances.reshape(X.shape[0],X.shape[1])
    #log_dens = log_dens.reshape(X.shape[0],X.shape[1])
    #pdb.set_trace()
    #Make a 3D plot
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.plot_surface(X, Y, 1 / distances ,cmap='viridis',linewidth=0)
    ax.scatter(Xdata[:,0],Xdata[:,1])
    ax.set_xlabel('X axis')
    ax.set_ylabel('Y axis')
    ax.set_zlabel('Z axis')
    ax.set_title(title)

def getfeatures (path, cl):
    X = []
    scl = 'c'+str(cl)
    for i in os.listdir(path):
        print("processing file ",i)
        if os.path.isfile(os.path.join(path,i)) and scl in i:
            X.append(getfeature(path,i))
    return np.asarray(X)
def getfeature(path,filename):
    im = cv2.imread(path+filename)
    imrgb = cv2.cvtColor(im, cv2.COLOR_BGR2RGB)
    R = imrgb[:,:,0]
    G = imrgb[:,:,1]
    B = imrgb[:,:,2]
    # normalize
    In= R+G+B
    mIn = np.max((In.flatten()))
    Rn = R/mIn
    Gn = G/mIn
    Bn = B/mIn
    # Compute mean of each component
    X = [np.mean(Rn.flatten()),np.mean(Gn.flatten()),np.mean(Bn.flatten())]
    return X

def main():
    training_images_path ='images/dataset1/app/'
    testing_images_path ='images/dataset1/test/'
    X1 = getfeatures(training_images_path, 1)
    # Keep 2 first components
    X1 = X1[:,0:2]
    X2 = getfeatures(training_images_path, 2)
    # Keep 2 first components
    X2 = X2[:,0:2]
    Xtest1 = getfeatures(testing_images_path, 1)
    Xtest1 = Xtest1[:,0:2]
    Xtest2 = getfeatures(testing_images_path, 2)
    Xtest2 = Xtest2[:,0:2]
    # train a classifier
    knn1 = NearestNeighbors(n_neighbors=1).fit(X1)
    knn2 = NearestNeighbors(n_neighbors=1).fit(X2)

    #pdb.set_trace()
    # display
    plt.plot(X1[:,0],X1[:,1],'+',label='Class 1')
    plt.plot(X2[:,0],X2[:,1],'o',label = 'Class2')
    plt.legend()
    plt.title('Training dataset: in feature space')
    plotKNN(knn1, X1,x_min=0, y_min=0, title='KDE pdf for Class 1')
    plt.plot
    plotKNN(knn2, X2,x_min=0, y_min=0, title='KDE pdf for Class 2')
    plotKNNdecision(knn1,knn2,X1,X2,x_min=0, y_min=0)
    plt.show()
if __name__ == "__main__":
    # execute only if run as a script
    main()
    

